"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var isEmpty_1 = __importDefault(require("lodash/isEmpty"));
var hash_sum_1 = __importDefault(require("hash-sum"));
var convert_hex_1 = __importDefault(require("./utilities/convert-hex"));
function castString(input) {
    if (!input && typeof input === 'boolean') {
        return 'false';
    }
    else if (input === 0) {
        return '0';
    }
    else if (input && typeof input === 'object') {
        if (isEmpty_1.default(input)) {
            return '';
        }
        else {
            return JSON.stringify(input);
        }
    }
    return input ? '' + input : '';
}
function castNumber(input) {
    var match = /\d+/.exec(input);
    var isDigitString = match ? match[0] === input : false;
    var isArray = Array.isArray(input);
    if (typeof input === 'string' && !isDigitString && input) {
        return convert_hex_1.default(hash_sum_1.default(input));
    }
    else if (isArray) {
        return input.reduce(function (a, b) { return castNumber(a) + castNumber(b); }, 0);
    }
    else if (input && typeof input === 'object' && !isArray) {
        return castNumber(Object.values(input));
    }
    else {
        return input ? +input : 0;
    }
}
function castBoolean(input) {
    if (typeof input === 'object' && isEmpty_1.default(input)) {
        return false;
    }
    else {
        return !!input;
    }
}
function castArray(input) {
    if (Array.isArray(input)) {
        return input;
    }
    else if (typeof input === 'object' && input) {
        return Object.values(input);
    }
    else if (input || typeof input === 'boolean' || input === 0) {
        return [input];
    }
    else {
        return [];
    }
}
function castObject(input, defaultKey) {
    var _a;
    var isArray = Array.isArray(input);
    if (!(input || typeof input === 'boolean' || input === 0)) {
        return {};
    }
    else if (typeof input === 'object' && !isArray) {
        return input;
    }
    else if (isArray) {
        return __assign({}, input);
    }
    else {
        return _a = {}, _a[defaultKey] = input, _a;
    }
}
var TypeInsurance = (function () {
    function TypeInsurance(input, options) {
        this.input = input;
        this.defaultKey = 'key';
        if (options) {
            this.defaultKey = castString(options.defaultKey);
        }
        this.string = this._string();
        this.number = this._number();
        this.boolean = this._boolean();
        this.array = this._array();
        this.object = this._object();
    }
    TypeInsurance.prototype._string = function () {
        var input = this.input;
        return castString(input);
    };
    TypeInsurance.prototype._number = function () {
        var input = this.input;
        return castNumber(input);
    };
    TypeInsurance.prototype._boolean = function () {
        var input = this.input;
        return castBoolean(input);
    };
    TypeInsurance.prototype._array = function () {
        var input = this.input;
        return castArray(input);
    };
    TypeInsurance.prototype._object = function () {
        var _a = this, input = _a.input, defaultKey = _a.defaultKey;
        return castObject(input, defaultKey);
    };
    return TypeInsurance;
}());
exports.default = TypeInsurance;
//# sourceMappingURL=index.js.map